/* --- Mini 4x Servo DCC Decoder by Philipp Gahtow ---
 * Email: digitalMoBa@arcor.de
 *
 * Expanded to support 12 servos on an Arduino Nano/Uno or even 16 servos on an Arduino Mega by Harald Sattler
 * Email: harry0001@satharo.de
 * 
 * Run Chip at 16Mhz with the standard Servo library, Fuses should not be touched (standard Arduino settings)
 * 
 * Activate Programming only at power up, when "LOCKCVPROG" is active, 
 * 
 * To program any of the CVs you have to power up the decoder with the button pressed.
 * Now the decoder is in programming mode and CVs can be written (precondition: #define UNLOCKEEPROM has to be enabled).
 * When updating any of the left/right end position the corresponding servo immediately runs to this newly defined position.
 * In CV prog mode the decoder also reacts on normal DCC telegrams.
 * After having done all CV settings cycle power (Off-On) - now the decoder is in operate mode.
 *  
// ATMEL ATMega328p / ARDUINO Nano
//
//                     +---\/---+
//  Komm. zum..  TX1  1|*       |30   Vin
//  ..Rechner    RX1  2|        |29   GND   
//               RST  3|        |28   RST   
//               GND  4|        |27   +5V   
//  DCC-Signal    D2  5|        |26   A7    (A6..A7 sind reine Analogeingänge, kein internal PullUp)
//  Servo1        D3  6|        |25   A6    
//  Servo2        D4  7|        |24   A5    Taster nach Masse
//  Servo3        D5  8|        |23   A4    (Servo16) Servos 13..16 nur mit AtMega 
//  Servo4        D6  9|        |22   A3    (Servo15) 
//  Servo5        D7 10|        |21   A2    (Servo14)  
//  Servo6        D8 11|        |20   A1    (Servo13)  
//  Servo7        D9 12|        |19   A0    Servo12  
//  Servo8       D10 13|        |18   AREF   
//  Servo9       D11 14|        |17   3V3    
//  Servo10      D12 15|        |16   D13   Servo11  
//                     +--------+
 *               
 *  CV# 6   (value * 10) = minimal Millis for all Servos (default = 54 -> 540 millis)
 *  CV# 7   (value * 10) = maximal Millis for all Servos (default = 240 -> 2400 millis)
 *                 
 * CV Setting for each Servo x = [1..4]:
 *  CV# x0   Adr high byte (0..255) - default: 0
 *  CV# x1   Adr low byte (0..255)  
 *  CV# x2   Position 1  (0..180) - default: 10
 *  CV# x3   Position 2  (0..180) - default: 170
 *  CV# x4   Active Time, if zero no power off! (0..255) - default: 62
 *  CV# x5   movement speed from fast(0) to slow(255) to Postion 1 - default: 1
 *  CV# x6   movement speed from fast(0) to slow(255) to Postion 2 - default: 1
 * 
 * ToDo: 
 *  - add CV read value on programming track
 * 
 * Harald Sattler, V2.0, 29.07.2019
 * - Umstellung auf Arduino Nano statt AtTiny85.
 * - Erweiterung auf 12 Servos, Arduino Nano passt auf die Konstruktion fuer den DCC-Weichendecoder "http://www.harald-sattler.de/html/dcc_weichendecoder.htm".
 *   Die Servo-Lib arbeitet nur mit 16bit Timern. Je 16bit Timer werden 12 Servos unterstuetzt. Der Nano hat einen 16bit Timer, somit ist bei 12 Servos Schluss.
 *   Bis zu 16 Servos werden nur mit Chips Mega oder STM32F1 unterstuetzt (gilt auch fuer die MobaTools Lib).
 *   Ich habe die defines und Routinen fuer die letzten 4 nicht unterstuetzten Servos drin gelassen, sie schaden nicht (funktionieren halt nicht). Wenn der Code
 *   mal auf einem Arduino Mega zu laufen kommt, sind dann 16 Servos moeglich. Ggf. muessen dann aber die Pin-Zuordnungen angepasst werden.
 * - Einige Ausgaben auf der seriellen SS (CV-Prog-Mode/Normal Mode). Im CV-Prog Mode werden beim PowerOn oder nach Reset die EEPROM-Inhalte der CVs ausgegeben.
 * - Die Umschaltung Normal/CV-Prog erfolgt über einen Taster an Pin A5 (der ist als Digital-IO mit internal PullUp konfiguriert, also kein Spannungsteiler notwendig).
 *   Ist der Taster beim PowerOn oder beim Reset gedrueckt, wird der CV-Prog Mode gestartet. 
 *   Ist der Taster beim PowerOn oder beim Reset nicht gedrueckt, geht der Decoder immer sofort in den Normal Mode.
 * Harald Sattler, V2.1, 01.08.2019
 * - ACK-Impuls-Erzeugung implementiert (Achtung: Die Werte werden nicht immer korrekt uebertragen, Lesefehler in JMRI DecoderPro)
 * Harald Sattler, V2.2, xx.xx.2019 (todo)
 * - Timeout beim CV-Programmieren.
 *   Die Endlagen der Servos werden mit Poti ueber die Base Station von DCC++ eingestellt. Da kein Auslesen der CVs realisiert ist, muessen die Werte immer 
 *   absolut eingegeben werden. Das bewirkt, dass oberer und unterer Endwert beim Beginn des Einstellens jeweils immer beim anderen Wert starten.
 *   Somit muessen bei jedem Einstellvorgang viele Werte uebersprungen werden, was entsprechend viele EEPROM-Schreibvorgaenge ausloesen wuerde.
 *   Um das zu verhindern wird das Schreiben ins EEPROM erst ausgeloest, wenn nach einer Aenderung 2-3 Sekunden vergangen sind.
 *	 Alternative: ACK-Impuls-Erzeugung ist implementiert -> Endpositionen vor Aenderungen aus dem EEPROM auslesen.
 */
 
// Version des Programms
#define VersionMajor 2
#define VersionMinor 1

// Servo Library:
#include "Servo.h"

//EEPROM Library:
#include <EEPROM.h>

// Serielle Schnittstelle
#include "HardwareSerial.h"
#include <stdio.h>				// for function sprintf

#define SHIFTADR				//Support shift by four (to use with IntelliBox Uhlenbrock or DCC++ by Gregg Berman)
#define LOCKCVPROG				//CV Programmmierung muss mit Taster aktiviert werden!
#define UNLOCKEEPROM			//schreiben im EEPROM möglich!

//###############################################################################

// Pins definieren
#define dccPIN 2				// attachInterrupt Funktion!
#define ServoPin1 3				// Servo 1
#define ServoPin2 4				// Servo 2
#define ServoPin3 5				// Servo 3
#define ServoPin4 6				// Servo 4
#define ServoPin5 7				// Servo 5
#define ServoPin6 8				// Servo 6
#define ServoPin7 9				// Servo 7
#define ServoPin8 10			// Servo 8
#define ServoPin9 11			// Servo 9
#define ServoPin10 12			// Servo 10
#define ServoPin11 13			// Servo 11
#define ServoPin12 A1			// Servo 12
#define ServoPin13 A2			// Servo 13
#define ServoPin14 A3			// Servo 14
#define ServoPin15 A4			// Servo 15
//#define ServoPin16 A6			// Servo 16
#define ButtonPin A7			// Taster nach Masse - Umschaltung in den CV-Prog-Mode

#define DccAckPin A0			// ACK-Impuls Ausgang


//Value Shift up:
#define ShiftAcTime 4			//Shift ActivTime
#define ShiftSpTime 0			//Shift Geschwindigkeit
#define MultiplicateMillis 10	//Multiplicate EEPROM Millis value

//###############################################################################
//Decoder Configuration:
#define EEPROMSHIFT 100			//for CV write/read!
#define EEPROMCVMinMillis 6		//minimal timing for Servo control
#define EEPROMCVMaxMillis 7		//maximal timing for Servo control

//------------ EEPROM Servo 1 --------------
#define EEPROMAdrH1 10			//Adresse High
#define EEPROMAdrL1 11			//Adresse Low
#define EEPROMP1S1 12			//Position 1
#define EEPROMP2S1 13			//Position 2
#define EEPROMAcS1 14			//ActivTime - Power off if in position when > 0
#define EEPROMSpP1S1 15			//Geschwindigkeit zu Position 1
#define EEPROMSpP2S1 16			//Geschwindigkeit zu Position 2
#define EEPROMPosS1 19			//aktuelle Position 0/1
//------------ EEPROM Servo 2 --------------
#define EEPROMAdrH2 20			//Adresse High
#define EEPROMAdrL2 21			//Adresse Low
#define EEPROMP1S2 22			//Position 1
#define EEPROMP2S2 23			//Position 2
#define EEPROMAcS2 24			//ActivTime
#define EEPROMSpP1S2 25			//Geschwindigkeit zu Position 1
#define EEPROMSpP2S2 26			//Geschwindigkeit zu Position 2
#define EEPROMPosS2 29			//aktuelle Position 0/1
//------------ EEPROM Servo 3 --------------
#define EEPROMAdrH3 30			//Adresse High
#define EEPROMAdrL3 31			//Adresse Low
#define EEPROMP1S3 32			//Position 1
#define EEPROMP2S3 33			//Position 2
#define EEPROMAcS3 34  			//ActivTime
#define EEPROMSpP1S3 35			//Geschwindigkeit zu Position 1
#define EEPROMSpP2S3 36			//Geschwindigkeit zu Position 2
#define EEPROMPosS3 39			//aktuelle Position 0/1
//------------ EEPROM Servo 4 --------------
#define EEPROMAdrH4 40			//Adresse High
#define EEPROMAdrL4 41			//Adresse Low
#define EEPROMP1S4 42			//Position 1
#define EEPROMP2S4 43			//Position 2
#define EEPROMAcS4 44			//ActivTime
#define EEPROMSpP1S4 45			//Geschwindigkeit zu Position 1
#define EEPROMSpP2S4 46			//Geschwindigkeit zu Position 2
#define EEPROMPosS4 49		//aktuelle Position 0/1
//------------ EEPROM Servo 5------------
#define EEPROMAdrH5 50			//Adresse High"
#define EEPROMAdrL5 51			//Adresse Low"
#define EEPROMP1S5 52			//Position 9"
#define EEPROMP2S5 53			//Position 10"
#define EEPROMAcS5 54			//ActivTime"
#define EEPROMSpP1S5 55			//Geschwindigkeit zu Position 9"
#define EEPROMSpP2S5 56			//Geschwindigkeit zu Position 10"
#define EEPROMPosS5 59			//aktuelle Position 0/1
//------------ EEPROM Servo 6------------
#define EEPROMAdrH6 60			//Adresse High"
#define EEPROMAdrL6 61			//Adresse Low"
#define EEPROMP1S6 62			//Position 11"
#define EEPROMP2S6 63			//Position 12"
#define EEPROMAcS6 64			//ActivTime"
#define EEPROMSpP1S6 65			//Geschwindigkeit zu Position 11"
#define EEPROMSpP2S6 66			//Geschwindigkeit zu Position 12"
#define EEPROMPosS6 69			//aktuelle Position 0/1
//------------ EEPROM Servo 7------------
#define EEPROMAdrH7 70			//Adresse High"
#define EEPROMAdrL7 71			//Adresse Low"
#define EEPROMP1S7 72			//Position 13"
#define EEPROMP2S7 73			//Position 14"
#define EEPROMAcS7 74			//ActivTime"
#define EEPROMSpP1S7 75			//Geschwindigkeit zu Position 13"
#define EEPROMSpP2S7 76			//Geschwindigkeit zu Position 14"
#define EEPROMPosS7 79			//aktuelle Position 0/1"
//------------ EEPROM Servo 8------------
#define EEPROMAdrH8 80			//Adresse High"
#define EEPROMAdrL8 81			//Adresse Low"
#define EEPROMP1S8 82			//Position 15"
#define EEPROMP2S8 83			//Position 16"
#define EEPROMAcS8 84			//ActivTime"
#define EEPROMSpP1S8 85			//Geschwindigkeit zu Position 15"
#define EEPROMSpP2S8 86			//Geschwindigkeit zu Position 16"
#define EEPROMPosS8 89			//aktuelle Position 0/1
//------------ EEPROM Servo 9------------
#define EEPROMAdrH9 90			//Adresse High"
#define EEPROMAdrL9 91			//Adresse Low"
#define EEPROMP1S9 92			//Position 17"
#define EEPROMP2S9 93			//Position 18"
#define EEPROMAcS9 94			//ActivTime"
#define EEPROMSpP1S9 95			//Geschwindigkeit zu Position 17"
#define EEPROMSpP2S9 96			//Geschwindigkeit zu Position 18"
#define EEPROMPosS9 99			//aktuelle Position 0/1
//------------ EEPROM Servo 10------------
#define EEPROMAdrH10 100		//Adresse High"
#define EEPROMAdrL10 101		//Adresse Low"
#define EEPROMP1S10 102			//Position 19"
#define EEPROMP2S10 103			//Position 20"
#define EEPROMAcS10 104			//ActivTime"
#define EEPROMSpP1S10 105		//Geschwindigkeit zu Position 19"
#define EEPROMSpP2S10 106		//Geschwindigkeit zu Position 20"
#define EEPROMPosS10 109		//aktuelle Position 0/1
//------------ EEPROM Servo 11------------
#define EEPROMAdrH11 110		//Adresse High"
#define EEPROMAdrL11 111		//Adresse Low"
#define EEPROMP1S11 112			//Position 21"
#define EEPROMP2S11 113			//Position 22"
#define EEPROMAcS11 114			//ActivTime"
#define EEPROMSpP1S11 115		//Geschwindigkeit zu Position 21"
#define EEPROMSpP2S11 116		//Geschwindigkeit zu Position 22"
#define EEPROMPosS11 119		//aktuelle Position 0/1
//------------ EEPROM Servo 12------------
#define EEPROMAdrH12 120		//Adresse High"
#define EEPROMAdrL12 121		//Adresse Low"
#define EEPROMP1S12 122			//Position 23"
#define EEPROMP2S12 123			//Position 24"
#define EEPROMAcS12 124			//ActivTime"
#define EEPROMSpP1S12 125		//Geschwindigkeit zu Position 23"
#define EEPROMSpP2S12 126		//Geschwindigkeit zu Position 24"
#define EEPROMPosS12 129		//aktuelle Position 0/1

// ab hier nur im Vorgriff auf Umstieg auf Arduino Mega
//------------ EEPROM Servo 13------------
#define EEPROMAdrH13 130		//Adresse High"
#define EEPROMAdrL13 131		//Adresse Low"
#define EEPROMP1S13 132			//Position 25"
#define EEPROMP2S13 133			//Position 26"
#define EEPROMAcS13 134			//ActivTime"
#define EEPROMSpP1S13 135		//Geschwindigkeit zu Position 25"
#define EEPROMSpP2S13 136		//Geschwindigkeit zu Position 26"
#define EEPROMPosS13 139		//aktuelle Position 0/1
//------------ EEPROM Servo 14------------
#define EEPROMAdrH14 140		//Adresse High"
#define EEPROMAdrL14 141		//Adresse Low"
#define EEPROMP1S14 142			//Position 27"
#define EEPROMP2S14 143			//Position 28"
#define EEPROMAcS14 144			//ActivTime"
#define EEPROMSpP1S14 145		//Geschwindigkeit zu Position 27"
#define EEPROMSpP2S14 146		//Geschwindigkeit zu Position 28"
#define EEPROMPosS14 149		//aktuelle Position 0/1
//------------ EEPROM Servo 15------------
#define EEPROMAdrH15 150		//Adresse High"
#define EEPROMAdrL15 151		//Adresse Low"
#define EEPROMP1S15 152			//Position 29"
#define EEPROMP2S15 153			//Position 30"
#define EEPROMAcS15 154			//ActivTime"
#define EEPROMSpP1S15 155		//Geschwindigkeit zu Position 29"
#define EEPROMSpP2S15 156		//Geschwindigkeit zu Position 30"
#define EEPROMPosS15 159		//aktuelle Position 0/1
//------------ EEPROM Servo 16------------
#define EEPROMAdrH16 160		//Adresse High"
#define EEPROMAdrL16 161		//Adresse Low"
#define EEPROMP1S16 162			//Position 31"
#define EEPROMP2S16 163			//Position 32"
#define EEPROMAcS16 164			//ActivTime"
#define EEPROMSpP1S16 165		//Geschwindigkeit zu Position 31"
#define EEPROMSpP2S16 166		//Geschwindigkeit zu Position 32"
#define EEPROMPosS16 169		//aktuelle Position 0/1
//###############################################################################

//default for Servo control:
unsigned int ServoMinMillis = 544;
unsigned int ServoMaxMillis = 2400;

// Servos definieren
Servo servo1;
Servo servo2;
Servo servo3;
Servo servo4;
Servo servo5;
Servo servo6;
Servo servo7;
Servo servo8;
Servo servo9;
Servo servo10;
Servo servo11;
Servo servo12;
Servo servo13;
Servo servo14;
Servo servo15;
Servo servo16;

unsigned int AdrServo1 = 1;
byte P1S1 = 10;				//Position 1
byte P2S1 = 170;			//Position 2
byte AcS1 = 5;				//ca. 1000ms aktiv (4 times shift left); no power off if zero!
byte PiS1 = 10;				//ist Position (value)
byte PosS1 = 0;				//aktuelle Position
byte SpP1S1 = 5;			//Geschwindigkeit zu Position 1
byte SpP2S1 = 5;			//Geschwindigkeit zu Position 2
unsigned int AdrServo2 = 2;
byte P1S2 = 10;
byte P2S2 = 170;
byte AcS2 = 5;
byte PiS2 = 10;
byte PosS2 = 0;
byte SpP1S2 = 5;
byte SpP2S2 = 5;
unsigned int AdrServo3 = 3;
byte P1S3 = 10;
byte P2S3 = 170;
byte AcS3 = 5;
byte PiS3 = 10;
byte PosS3 = 0;
byte SpP1S3 = 5;
byte SpP2S3 = 5;
unsigned int AdrServo4 = 4;
byte P1S4 = 10;
byte P2S4 = 170;
byte AcS4 = 5;
byte PiS4 = 10;
byte PosS4 = 0;
byte SpP1S4 = 5;
byte SpP2S4 = 5;
unsigned int AdrServo5 = 5;
byte P1S5 = 10;
byte P2S5 = 170;
byte AcS5 = 5;
byte PiS5 = 10;
byte PosS5 = 0;
byte SpP1S5 = 5;
byte SpP2S5 = 5;
unsigned int AdrServo6 = 6;
byte P1S6 = 10;
byte P2S6 = 170;
byte AcS6 = 5;
byte PiS6 = 10;
byte PosS6 = 0;
byte SpP1S6 = 5;
byte SpP2S6 = 5;
unsigned int AdrServo7 = 7;
byte P1S7 = 10;
byte P2S7 = 170;
byte AcS7 = 5;
byte PiS7 = 10;
byte PosS7 = 0;
byte SpP1S7 = 5;
byte SpP2S7 = 5;
unsigned int AdrServo8 = 8;
byte P1S8 = 10;
byte P2S8 = 170;
byte AcS8 = 5;
byte PiS8 = 10;
byte PosS8 = 0;
byte SpP1S8 = 5;
byte SpP2S8 = 5;
unsigned int AdrServo9 = 9;
byte P1S9 = 10;
byte P2S9 = 170;
byte AcS9 = 5;
byte PiS9 = 10;
byte PosS9 = 0;
byte SpP1S9 = 5;
byte SpP2S9 = 5;
unsigned int AdrServo10 = 10;
byte P1S10 = 10;
byte P2S10 = 170;
byte AcS10  = 5;
byte PiS10 = 10;
byte PosS10 = 0;
byte SpP1S10 = 5;
byte SpP2S10 = 5;
unsigned int AdrServo11 = 11;
byte P1S11 = 10;
byte P2S11 = 170;
byte AcS11 = 5;
byte PiS11 = 10;
byte PosS11 = 0;
byte SpP1S11 = 5;
byte SpP2S11 = 5;
unsigned int AdrServo12 = 12;
byte P1S12 = 10;
byte P2S12 = 170;
byte AcS12 = 5;
byte PiS12 = 10;
byte PosS12 = 0;
byte SpP1S12 = 5;
byte SpP2S12 = 5;
unsigned int AdrServo13 = 13;
byte P1S13 = 10;
byte P2S13 = 170;
byte AcS13 = 5;
byte PiS13 = 10;
byte PosS13 = 0;
byte SpP1S13 = 5;
byte SpP2S13 = 5;
unsigned int AdrServo14 = 14;
byte P1S14 = 10;
byte P2S14 = 170;
byte AcS14 = 5;
byte PiS14 = 10;
byte PosS14 = 0;
byte SpP1S14 = 5;
byte SpP2S14 = 5;
unsigned int AdrServo15 = 15;
byte P1S15 = 10;
byte P2S15 = 170;
byte AcS15 = 5;
byte PiS15 = 10;
byte PosS15 = 0;
byte SpP1S15 = 5;
byte SpP2S15 = 5;
unsigned int AdrServo16 = 16;
byte P1S16 = 10;
byte P2S16 = 170;
byte AcS16 = 5;
byte PiS16 = 10;
byte PosS16 = 0;
byte SpP1S16 = 5;
byte SpP2S16 = 5;

#if defined(LOCKCVPROG)
byte ProgMode = 0x00;
#endif

//***********************************************************
//Variablen für DCC Erkennung
byte countone = 0;    //Zähle die high so das man eine Präambel erkennen kann (min. 10 high)
boolean getdata = false;     //Es werden DCC Daten gelesen und im Array data abgespeichert
byte data[4];                //eingelesene Byte-Werte nach einer Präambel
byte datalength = 0;  //Position in data wo gerade der nächste Wert hinkommt
byte countbit = 0;    //Zähle Bits, nach 8 Bit wurde ein Byte gelesen und setzte dann die Variable auf 0
byte dxor = 0;        //xor Prüfsumme
unsigned int dccAdr = 0;          //Adresse
byte dccType = 0xFF;  //Art der DCC Daten (Long, Short, Accessory)
#define DCC_SHORT_ADDRESS           0x00
#define DCC_LONG_ADDRESS            0x01
#define DCC_ACC_ADDRESS             0x02

bool readone = false;
bool readzero = false;

unsigned long micro = 0;
unsigned long DCCMicros = 0;    //Time to check DCC Data "1" or "0"
//------------------------------------------------------------------------

unsigned long timeservo1 = 0;
unsigned long timeservo2 = 0;
unsigned long timeservo3 = 0;
unsigned long timeservo4 = 0;
unsigned long timeservo5 = 0;
unsigned long timeservo6 = 0;
unsigned long timeservo7 = 0;
unsigned long timeservo8 = 0;
unsigned long timeservo9 = 0;
unsigned long timeservo10 = 0;
unsigned long timeservo11 = 0;
unsigned long timeservo12 = 0;
unsigned long timeservo13 = 0;
unsigned long timeservo14 = 0;
unsigned long timeservo15 = 0;
unsigned long timeservo16 = 0;

void setup() {

  Serial.begin(115200);	// serielle SS initialsieren

  EEPROMReadToRam();  //Read values for Servo driver
  InitServo(0);    //Servos set to last position!

  // Taster-Anschluss initialisieren
  pinMode(ButtonPin, INPUT_PULLUP);

  // ACK-Pin initialisieren
  pinMode(DccAckPin, OUTPUT);

  //Change Mode Normal/Programmierung
  #if defined(LOCKCVPROG)
  //Aktivierung der CV Programmierung?
  //if (analogRead(ButtonPin) < 600) {  //default 900
  if (digitalRead(ButtonPin) == 0) {
	ProgMode = 0x01;  //active
	Serial.println("CV-Prog-Mode aktiv -> Schalter RECHTS");
	dumpEEPROM();	// relevanten EEPROM-Inhalt auf SS ausgeben
  }
  else {
    ProgMode = 0;   //inactive
	Serial.println("Normal-Mode aktiv -> Schalter LINKS");
  }
  #endif

  //pinMode(dccPIN, INPUT); // Interrupt 0, use external pull up resistor
  attachInterrupt(0, dccdata, CHANGE);  //ISR für den Dateneingang
   
  DCCMicros = micros();
}

void loop() {

	// Testroutine
	//static unsigned long testtimer = 0;
	//if (millis() > testtimer + 100)
	//{
	//	notifyCVAck();
	//	testtimer = millis();
	//}

  //decode received DCC Data!
  dccupdatedata();
  
  //-------------------- SERVO 1 ----------------------------
  if (PosS1 == 1) { //Servo 1 soll in Position 1 stehen!
    if (PiS1 != P1S1) {
      if (millis() - timeservo1 > (SpP1S1 << ShiftSpTime)) {
        //move
        PiS1 = getNextStep(PiS1, P1S1);
        servo1.write(PiS1);
        timeservo1 = millis();
      }
    }
  }
  else { //Servo 1 soll in Position 2 stehen!
    if (PiS1 != P2S1) {
      if (millis() - timeservo1 > (SpP2S1 << ShiftSpTime)) {
        //move
        PiS1 = getNextStep(PiS1, P2S1);
        servo1.write(PiS1);
        timeservo1 = millis();
      }
    }
  }
  //-------------------- SERVO 2 ----------------------------
  if (PosS2 == 1) { //Servo 2 soll in Position 1 stehen!
    if (PiS2 != P1S2) {
      if (millis() - timeservo2 > (SpP1S2 << ShiftSpTime)) {
        //move
        PiS2 = getNextStep(PiS2, P1S2);
        servo2.write(PiS2);
        timeservo2 = millis();
      }
    }
  }
  else { //Servo 2 soll in Position 2 stehen!
    if (PiS2 != P2S2) {
      if (millis() - timeservo2 > (SpP2S2 << ShiftSpTime)) {
        //move
        PiS2 = getNextStep(PiS2, P2S2);
        servo2.write(PiS2);
        timeservo2 = millis();
      }
    }
  }
  //-------------------- SERVO 3 ----------------------------
  if (PosS3 == 1) { //Servo 3 soll in Position 1 stehen!
    if (PiS3 != P1S3) {
      if (millis() - timeservo3 > (SpP1S3 << ShiftSpTime)) {
        //move
        PiS3 = getNextStep(PiS3, P1S3);
        servo3.write(PiS3);
        timeservo3 = millis();
      }
    }
  }
  else { //Servo 3 soll in Position 2 stehen!
    if (PiS3 != P2S3) {
      if (millis() - timeservo3 > (SpP2S3 << ShiftSpTime)) {
        //move
        PiS3 = getNextStep(PiS3, P2S3);
        servo3.write(PiS3);
        timeservo3 = millis();
      }
    }
  }
  //-------------------- SERVO 4 ----------------------------
  if (PosS4 == 1) { //Servo 4 soll in Position 1 stehen!
    if (PiS4 != P1S4) {
      if (millis() - timeservo4 > (SpP1S4 << ShiftSpTime)) {
        //move
        PiS4 = getNextStep(PiS4, P1S4);
        servo4.write(PiS4);
        timeservo4 = millis();
      }
    }
  }
  else { //Servo 4 soll in Position 2 stehen!
    if (PiS4 != P2S4) {
      if (millis() - timeservo4 > (SpP2S4 << ShiftSpTime)) {
        //move
        PiS4 = getNextStep(PiS4, P2S4);
        servo4.write(PiS4);
        timeservo4 = millis();
      }
    }
  }
  //-------------------- SERVO 5 ----------------------------
  if (PosS5 == 1) { //Servo 5 soll in Position 1 stehen!
	  if (PiS5 != P1S5) {
		  if (millis() - timeservo5 > (SpP1S5 << ShiftSpTime)) {
			  //move
			  PiS5 = getNextStep(PiS5, P1S5);
			  servo5.write(PiS5);
			  timeservo5 = millis();
		  }
	  }
  }
  else { //Servo 5 soll in Position 2 stehen!
	  if (PiS5 != P2S5) {
		  if (millis() - timeservo5 > (SpP2S5 << ShiftSpTime)) {
			  //move
			  PiS5 = getNextStep(PiS5, P2S5);
			  servo5.write(PiS5);
			  timeservo5 = millis();
		  }
	  }
  }
  //-------------------- SERVO 6 ----------------------------
  if (PosS6 == 1) { //Servo 6 soll in Position 1 stehen!
	  if (PiS6 != P1S6) {
		  if (millis() - timeservo6 > (SpP1S6 << ShiftSpTime)) {
			  //move
			  PiS6 = getNextStep(PiS6, P1S6);
			  servo6.write(PiS6);
			  timeservo6 = millis();
		  }
	  }
  }
  else { //Servo 6 soll in Position 2 stehen!
	  if (PiS6 != P2S6) {
		  if (millis() - timeservo6 > (SpP2S6 << ShiftSpTime)) {
			  //move
			  PiS6 = getNextStep(PiS6, P2S6);
			  servo6.write(PiS6);
			  timeservo6 = millis();
		  }
	  }
  }
  //-------------------- SERVO 7 ----------------------------
  if (PosS7 == 1) { //Servo 7 soll in Position 1 stehen!
	  if (PiS7 != P1S7) {
		  if (millis() - timeservo7 > (SpP1S7 << ShiftSpTime)) {
			  //move
			  PiS7 = getNextStep(PiS7, P1S7);
			  servo7.write(PiS7);
			  timeservo7 = millis();
		  }
	  }
  }
  else { //Servo 7 soll in Position 2 stehen!
	  if (PiS7 != P2S7) {
		  if (millis() - timeservo7 > (SpP2S7 << ShiftSpTime)) {
			  //move
			  PiS7 = getNextStep(PiS7, P2S7);
			  servo7.write(PiS7);
			  timeservo7 = millis();
		  }
	  }
  }
  //-------------------- SERVO 8 ----------------------------
  if (PosS8 == 1) { //Servo 8 soll in Position 1 stehen!
	  if (PiS8 != P1S8) {
		  if (millis() - timeservo8 > (SpP1S8 << ShiftSpTime)) {
			  //move
			  PiS8 = getNextStep(PiS8, P1S8);
			  servo8.write(PiS8);
			  timeservo8 = millis();
		  }
	  }
  }
  else { //Servo 8 soll in Position 2 stehen!
	  if (PiS8 != P2S8) {
		  if (millis() - timeservo8 > (SpP2S8 << ShiftSpTime)) {
			  //move
			  PiS8 = getNextStep(PiS8, P2S8);
			  servo8.write(PiS8);
			  timeservo8 = millis();
		  }
	  }
  }
  //-------------------- SERVO 9 ----------------------------
  if (PosS9 == 1) { //Servo 9 soll in Position 1 stehen!
	  if (PiS9 != P1S9) {
		  if (millis() - timeservo9 > (SpP1S9 << ShiftSpTime)) {
			  //move
			  PiS9 = getNextStep(PiS9, P1S9);
			  servo9.write(PiS9);
			  timeservo9 = millis();
		  }
	  }
  }
  else { //Servo 9 soll in Position 2 stehen!
	  if (PiS9 != P2S9) {
		  if (millis() - timeservo9 > (SpP2S9 << ShiftSpTime)) {
			  //move
			  PiS9 = getNextStep(PiS9, P2S9);
			  servo9.write(PiS9);
			  timeservo9 = millis();
		  }
	  }
  }
  //-------------------- SERVO 10 ----------------------------
  if (PosS10 == 1) { //Servo 10 soll in Position 1 stehen!
	  if (PiS10 != P1S10) {
		  if (millis() - timeservo10 > (SpP1S10 << ShiftSpTime)) {
			  //move
			  PiS10 = getNextStep(PiS10, P1S10);
			  servo10.write(PiS10);
			  timeservo10 = millis();
		  }
	  }
  }
  else { //Servo 10 soll in Position 2 stehen!
	  if (PiS10 != P2S10) {
		  if (millis() - timeservo10 > (SpP2S10 << ShiftSpTime)) {
			  //move
			  PiS10 = getNextStep(PiS10, P2S10);
			  servo10.write(PiS10);
			  timeservo10 = millis();
		  }
	  }
  }
  //-------------------- SERVO 11 ----------------------------
  if (PosS11 == 1) { //Servo 11 soll in Position 1 stehen!
	  if (PiS11 != P1S11) {
		  if (millis() - timeservo11 > (SpP1S11 << ShiftSpTime)) {
			  //move
			  PiS11 = getNextStep(PiS11, P1S11);
			  servo11.write(PiS11);
			  timeservo11 = millis();
		  }
	  }
  }
  else { //Servo 11 soll in Position 2 stehen!
	  if (PiS11 != P2S11) {
		  if (millis() - timeservo11 > (SpP2S11 << ShiftSpTime)) {
			  //move
			  PiS11 = getNextStep(PiS11, P2S11);
			  servo11.write(PiS11);
			  timeservo11 = millis();
		  }
	  }
  }
  //-------------------- SERVO 12 ----------------------------
  if (PosS12 == 1) { //Servo 12 soll in Position 1 stehen!
	  if (PiS12 != P1S12) {
		  if (millis() - timeservo12 > (SpP1S12 << ShiftSpTime)) {
			  //move
			  PiS12 = getNextStep(PiS12, P1S12);
			  servo12.write(PiS12);
			  timeservo12 = millis();
		  }
	  }
  }
  else { //Servo 12 soll in Position 2 stehen!
	  if (PiS12 != P2S12) {
		  if (millis() - timeservo12 > (SpP2S12 << ShiftSpTime)) {
			  //move
			  PiS12 = getNextStep(PiS12, P2S12);
			  servo12.write(PiS12);
			  timeservo12 = millis();
		  }
	  }
  }
  //-------------------- SERVO 13 ----------------------------
  if (PosS13 == 1) { //Servo 13 soll in Position 1 stehen!
	  if (PiS13 != P1S13) {
		  if (millis() - timeservo13 > (SpP1S13 << ShiftSpTime)) {
			  //move
			  PiS13 = getNextStep(PiS13, P1S13);
			  servo13.write(PiS13);
			  timeservo13 = millis();
		  }
	  }
  }
  else { //Servo 13 soll in Position 2 stehen!
	  if (PiS13 != P2S13) {
		  if (millis() - timeservo13 > (SpP2S13 << ShiftSpTime)) {
			  //move
			  PiS13 = getNextStep(PiS13, P2S13);
			  servo13.write(PiS13);
			  timeservo13 = millis();
		  }
	  }
  }
  //-------------------- SERVO 14 ----------------------------
  if (PosS14 == 1) { //Servo 14 soll in Position 1 stehen!
	  if (PiS14 != P1S14) {
		  if (millis() - timeservo14 > (SpP1S14 << ShiftSpTime)) {
			  //move
			  PiS14 = getNextStep(PiS14, P1S14);
			  servo14.write(PiS14);
			  timeservo14 = millis();
		  }
	  }
  }
  else { //Servo 14 soll in Position 2 stehen!
	  if (PiS14 != P2S14) {
		  if (millis() - timeservo14 > (SpP2S14 << ShiftSpTime)) {
			  //move
			  PiS14 = getNextStep(PiS14, P2S14);
			  servo14.write(PiS14);
			  timeservo14 = millis();
		  }
	  }
  }
  //-------------------- SERVO 15 ----------------------------
  if (PosS15 == 1) { //Servo 15 soll in Position 1 stehen!
	  if (PiS15 != P1S15) {
		  if (millis() - timeservo15 > (SpP1S15 << ShiftSpTime)) {
			  //move
			  PiS15 = getNextStep(PiS15, P1S15);
			  servo15.write(PiS15);
			  timeservo15 = millis();
		  }
	  }
  }
  else { //Servo 15 soll in Position 2 stehen!
	  if (PiS15 != P2S15) {
		  if (millis() - timeservo15 > (SpP2S15 << ShiftSpTime)) {
			  //move
			  PiS15 = getNextStep(PiS15, P2S15);
			  servo15.write(PiS15);
			  timeservo15 = millis();
		  }
	  }
  }
  //-------------------- SERVO 16 ----------------------------
  if (PosS16 == 1) { //Servo 16 soll in Position 1 stehen!
	  if (PiS16 != P1S16) {
		  if (millis() - timeservo16 > (SpP1S16 << ShiftSpTime)) {
			  //move
			  PiS16 = getNextStep(PiS16, P1S16);
			  servo16.write(PiS16);
			  timeservo16 = millis();
		  }
	  }
  }
  else { //Servo 16 soll in Position 2 stehen!
	  if (PiS16 != P2S16) {
		  if (millis() - timeservo16 > (SpP2S16 << ShiftSpTime)) {
			  //move
			  PiS16 = getNextStep(PiS16, P2S16);
			  servo16.write(PiS16);
			  timeservo16 = millis();
		  }
	  }
  }


  //switch servo off if not in movement!
  if (((millis() - timeservo1) >= (AcS1 << ShiftAcTime)) && (AcS1 != 0))
    servo1.detach();
  if (((millis() - timeservo2) >= (AcS2 << ShiftAcTime)) && (AcS2 != 0))
    servo2.detach();  
  if (((millis() - timeservo3) >= (AcS3 << ShiftAcTime)) && (AcS3 != 0))
    servo3.detach();  
  if (((millis() - timeservo4) >= (AcS4 << ShiftAcTime)) && (AcS4 != 0))
	  servo4.detach();
  if (((millis() - timeservo5) >= (AcS5 << ShiftAcTime)) && (AcS5 != 0))
	  servo5.detach();
  if (((millis() - timeservo6) >= (AcS6 << ShiftAcTime)) && (AcS6 != 0))
	  servo6.detach();
  if (((millis() - timeservo7) >= (AcS7 << ShiftAcTime)) && (AcS7 != 0))
	  servo7.detach();
  if (((millis() - timeservo8) >= (AcS8 << ShiftAcTime)) && (AcS8 != 0))
	  servo8.detach();
  if (((millis() - timeservo9) >= (AcS9 << ShiftAcTime)) && (AcS9 != 0))
	  servo9.detach();
  if (((millis() - timeservo10) >= (AcS10 << ShiftAcTime)) && (AcS10 != 0))
	  servo10.detach();
  if (((millis() - timeservo11) >= (AcS11 << ShiftAcTime)) && (AcS11 != 0))
	  servo11.detach();
  if (((millis() - timeservo12) >= (AcS12 << ShiftAcTime)) && (AcS12 != 0))
	  servo12.detach();
  if (((millis() - timeservo13) >= (AcS13 << ShiftAcTime)) && (AcS13 != 0))
	  servo13.detach();
  if (((millis() - timeservo14) >= (AcS14 << ShiftAcTime)) && (AcS14 != 0))
	  servo14.detach();
  if (((millis() - timeservo15) >= (AcS15 << ShiftAcTime)) && (AcS15 != 0))
	  servo15.detach();
  if (((millis() - timeservo16) >= (AcS16 << ShiftAcTime)) && (AcS16 != 0))
	  servo16.detach();
}

//--------------------------------------------------------------------------------
//Interrupt funktion auf dccPin
void dccdata() {
  micro = micros();
  if (digitalRead(dccPIN) == HIGH) {
    DCCMicros = micro;    //16MHz = resolution of four microseconds
  }
  else {
    if (micro <= (DCCMicros + 70)) {      //1-Bit gelesen; over 68us
      readone = true;
    }
    else {                  //0-Bit gelesen
      readzero = true;
    }  
  }
  bitSet(EIFR,INTF0);   //IRQ-Bit des external int löschen
  //auf der fallenden Flanke kommen sonst 2 IRQ wegen "Prellen" des Signals!
}

//--------------------------------------------------------------------------------
//need to update inside each loop
void dccupdatedata() {
  if (readone) {  //EINS-Bit
    readone = false;
    if (getdata == true) 
      countbit += 1;    //Abzählen der Bytes, also jeweils 8 Bit
      
    countone += 1;            //Zählen der 1-Bit für Präambel erforderlich
    if (getdata == true && countbit <= 8) {    //eingelesenen Bitwert im Array Speichern
      bitWrite(data[datalength], 8-countbit, 1);    //Speichert das ein 1-Bit gelesen wurde
    }
    if (countbit > 8) {        //End-Bit gelesen.
      countbit = 0; 
      countone = 0;      
      getdata = false;        //Stop des Einlesen der Daten
      //XOR Prüfen:
      if (data[datalength] == dxor) {  //Prüfen von XOR und letztes Byte
        dccauswertung();         //Fertig, Daten Auswerten!
      }
    }
  }  //Ende 1-Bit
  if (readzero) { //ZERO-Bit
    readzero = false;
    if (getdata == true) 
      countbit += 1;    //Abzählen der Bytes, also jeweils 8 Bit
      
    if (getdata == true && countbit <= 8) {    //eingelesenen Bitwert im Array Speichern
      bitWrite(data[datalength], 8-countbit, 0);    //Speichert das ein 0-Bit gelesen wurde
    }
    if (countone > 10) {   //Präambel erkannt ab hier Daten lesen. (Es wurden mind. 10 HIGH erkannt)
      getdata = true;     //Einlesen der Bitwerte in Array aktivieren.
      datalength = 0;    //Position im Array an der die Bitwerte gespeichert werden.
      countbit = 0;       //beginne Bits zu zählen. Auswertung von 1 Byte Blöcken ermöglichen nach 8 Bit.
      dxor = 0;          //XOR zurücksetzten
    }
    if (countbit > 8) {    //Null-Bit gelesen. (Immer nach 1 Byte)
      countbit = 0;
      dxor = dxor ^ data[datalength];  //XOR bestimmen!
      datalength += 1;  //Bitweise im Array weitergehen.
    }
    countone = 0;    //Es wurde ein 0-Bit gelesen, lösche Anzahl gelesener 1-Bit
  }  //Ende 0-Bit
}

//--------------------------------------------------------------------------------
//Auswerten der eingelesenen Bitwerte.
void dccauswertung() {
  /*
    Beschreibung der Einträge im data Array:
      data[0]   : 1. Byte nach der Präambel (Adresse)
      Bit 8     : Null-Bit
      data[1]   : 2. Byte (long Adresse oder Daten)
      Bit 17    : Null-Bit
      data[2]   : 3. Byte (bei datalength == 26 => CRC)
      Bit 26    : Null-Bit (bei datalength == 26 => End-Bit)
      data[3]   : 4. Byte (wenn vorhanden, long Adr. oder long Data)
      Bit 35    : End-Bit (wenn vorhanden, long Adr. oder long Data)
      data[4]   : 5. Byte (wenn vorhanden, long Adr. und long Data)
      Bit 44    : End-Bit (wenn vorhanden, long Adr. und long Data)
  */

  /*Instructions packets using Direct CV Addressing are 4 byte packets of the format:
    long-preamble 0 0111CCAA 0 AAAAAAAA 0 DDDDDDDD 0 EEEEEEEE 1
      CC=10 Bit Manipulation
      CC=01 Verify byte
      CC=11 Write byte  */
      
  //CV write byte: ---------------------------
  //Long-preamble   0  011111AA  0  AAAAAAAA  0  DDDDDDDD  0  EEEEEEEE  1 
  if ((data[0] >> 2) == B00011111) {
    #if defined(LOCKCVPROG)
    if (ProgMode > 0) {
    #endif  
    #if defined(UNLOCKEEPROM)
      uint16_t CV_Adr = (((data[0] & 0x03) << 8) + data[1] + 1);
      //Change Value inside EEPROM:
      EEPROM.update(CV_Adr+EEPROMSHIFT, data[2]); //save value in EEPROM

	  // Ausgabe des DCC-Befehls auf die serielle SS
	  Serial.print("CV-Prog Mode - CV: "); Serial.print(CV_Adr);
	  Serial.print("     Wert: "); Serial.println(data[2]);
	  
      //Check if Servo is in Program Position:
      uint8_t SelServo = 0; //select servo = none
      //Servo1:----------------------------------
      if (CV_Adr == EEPROMP1S1) {   //Servo1; Pos1
        SelServo = 1;
        EEPROM.update(EEPROMPosS1+EEPROMSHIFT, 1);
      }
      if (CV_Adr == EEPROMP2S1) {   //Servo1; Pos2
        SelServo = 1;
        EEPROM.update(EEPROMPosS1+EEPROMSHIFT, 0);
      }  
      //Servo2:----------------------------------
      if (CV_Adr == EEPROMP1S2) {   //Servo2; Pos1
        SelServo = 2;
        EEPROM.update(EEPROMPosS2+EEPROMSHIFT, 1);
      }
      if (CV_Adr == EEPROMP2S2) {   //Servo2; Pos2
        SelServo = 2;
        EEPROM.update(EEPROMPosS2+EEPROMSHIFT, 0);
      }
      //Servo3:----------------------------------
      if (CV_Adr == EEPROMP1S3) {   //Servo3; Pos1
        SelServo = 3;
        EEPROM.update(EEPROMPosS3+EEPROMSHIFT, 1);
      }
      if (CV_Adr == EEPROMP2S3) {   //Servo3; Pos2
        SelServo = 3;
        EEPROM.update(EEPROMPosS3+EEPROMSHIFT, 0);
      }
      //Servo4:----------------------------------
      if (CV_Adr == EEPROMP1S4) {   //Servo4; Pos1
        SelServo = 4;
        EEPROM.update(EEPROMPosS4+EEPROMSHIFT, 1);
      }
      if (CV_Adr == EEPROMP2S4) {   //Servo4; Pos2
        SelServo = 4;
        EEPROM.update(EEPROMPosS4+EEPROMSHIFT, 0);
      }


	  //Servo5:----------------------------------
	  if (CV_Adr == EEPROMP1S5) {   //Servo5; Pos1
		  SelServo = 5;
		  EEPROM.update(EEPROMPosS5 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S5) {   //Servo5; Pos2
		  SelServo = 5;
		  EEPROM.update(EEPROMPosS5 + EEPROMSHIFT, 0);
	  }
	  //Servo6:----------------------------------
	  if (CV_Adr == EEPROMP1S6) {   //Servo6; Pos1
		  SelServo = 6;
		  EEPROM.update(EEPROMPosS6 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S6) {   //Servo6; Pos2
		  SelServo = 6;
		  EEPROM.update(EEPROMPosS6 + EEPROMSHIFT, 0);
	  }
	  //Servo7:----------------------------------
	  if (CV_Adr == EEPROMP1S7) {   //Servo7; Pos1
		  SelServo = 7;
		  EEPROM.update(EEPROMPosS7 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S7) {   //Servo7; Pos2
		  SelServo = 7;
		  EEPROM.update(EEPROMPosS7 + EEPROMSHIFT, 0);
	  }
	  //Servo8:----------------------------------
	  if (CV_Adr == EEPROMP1S8) {   //Servo8; Pos1
		  SelServo = 8;
		  EEPROM.update(EEPROMPosS8 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S8) {   //Servo8; Pos2
		  SelServo = 8;
		  EEPROM.update(EEPROMPosS8 + EEPROMSHIFT, 0);
	  }
	  //Servo9:----------------------------------
	  if (CV_Adr == EEPROMP1S9) {   //Servo9; Pos1
		  SelServo = 9;
		  EEPROM.update(EEPROMPosS9 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S9) {   //Servo9; Pos2
		  SelServo = 9;
		  EEPROM.update(EEPROMPosS9 + EEPROMSHIFT, 0);
	  }
	  //Servo10:----------------------------------
	  if (CV_Adr == EEPROMP1S10) {   //Servo10; Pos1
		  SelServo = 10;
		  EEPROM.update(EEPROMPosS10 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S10) {   //Servo10; Pos2
		  SelServo = 10;
		  EEPROM.update(EEPROMPosS10 + EEPROMSHIFT, 0);
	  }
	  //Servo11:----------------------------------
	  if (CV_Adr == EEPROMP1S11) {   //Servo11; Pos1
		  SelServo = 11;
		  EEPROM.update(EEPROMPosS11 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S11) {   //Servo11; Pos2
		  SelServo = 11;
		  EEPROM.update(EEPROMPosS11 + EEPROMSHIFT, 0);
	  }
	  //Servo12:----------------------------------
	  if (CV_Adr == EEPROMP1S12) {   //Servo12; Pos1
		  SelServo = 12;
		  EEPROM.update(EEPROMPosS12 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S12) {   //Servo12; Pos2
		  SelServo = 12;
		  EEPROM.update(EEPROMPosS12 + EEPROMSHIFT, 0);
	  }
	  //Servo13:----------------------------------
	  if (CV_Adr == EEPROMP1S13) {   //Servo13; Pos1
		  SelServo = 13;
		  EEPROM.update(EEPROMPosS13 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S13) {   //Servo13; Pos2
		  SelServo = 13;
		  EEPROM.update(EEPROMPosS13 + EEPROMSHIFT, 0);
	  }
	  //Servo14:----------------------------------
	  if (CV_Adr == EEPROMP1S14) {   //Servo14; Pos1
		  SelServo = 14;
		  EEPROM.update(EEPROMPosS14 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S14) {   //Servo14; Pos2
		  SelServo = 14;
		  EEPROM.update(EEPROMPosS14 + EEPROMSHIFT, 0);
	  }
	  //Servo15:----------------------------------
	  if (CV_Adr == EEPROMP1S15) {   //Servo15; Pos1
		  SelServo = 15;
		  EEPROM.update(EEPROMPosS15 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S15) {   //Servo15; Pos2
		  SelServo = 15;
		  EEPROM.update(EEPROMPosS15 + EEPROMSHIFT, 0);
	  }
	  //Servo16:----------------------------------
	  if (CV_Adr == EEPROMP1S16) {   //Servo16; Pos1
		  SelServo = 16;
		  EEPROM.update(EEPROMPosS16 + EEPROMSHIFT, 1);
	  }
	  if (CV_Adr == EEPROMP2S16) {   //Servo16; Pos2
		  SelServo = 16;
		  EEPROM.update(EEPROMPosS16 + EEPROMSHIFT, 0);
	  }

	  
	  //Read Values new to RAM:
      EEPROMReadToRam();  //update RAM values
      //Show up new position:
      InitServo(SelServo);  //set new position to servo
    #endif  
    #if defined(LOCKCVPROG)  
      }
    #endif
  }
///*
  //CV verify byte: ---------------------------
  //Long-preamble   0  011101AA  0  AAAAAAAA  0  DDDDDDDD  0  EEEEEEEE  1 
  if ((data[0] >> 2) == B00011101 && ProgMode > 0) {
    byte verify = EEPROM.read(((data[0] & 0x03) << 8) + data[1] + 1 + EEPROMSHIFT);

	// Ausgabe des DCC-Befehls auf die serielle SS
	Serial.print("Verify Byte - Adr: "); Serial.print((((data[0] & 0x03) << 8) + data[1] + 1)+ EEPROMSHIFT);
	Serial.print("     Data: "); Serial.println(data[2]);

    if (verify == data[2]) {
		notifyCVAck();
		//servo1.attach(ServoPin1,ServoMinMillis,ServoMaxMillis);    //attach the servo to pin
		//if (PiS1 != P1S1)
		//	servo1.write(P1S1);
		//else servo1.write(P2S1);  
		//delay(50);
		//servo1.write(PiS1);  
	}
  }
  //CV read/verify/write bit: -----------------------------
  //long-preamble   0  011110AA  0  AAAAAAAA  0  111KDBBB  0  EEEEEEEE  1
  //BBB represents the bit position 
  //D contains the value of the bit to be verified or written
  //K=1 signifies a "Write Bit" operation and K=0 signifies a "Bit Verify"  
  if ((data[0] >> 2) == B00011110 && ProgMode > 0) {
    byte CVadr = ((data[0] & 0x03) << 8) + data[1] + 1 + EEPROMSHIFT;
    byte readbyte = EEPROM.read(CVadr);
    if (data[2] >> 4 == B1110) {  //Verify! Bit

	  // Ausgabe des DCC-Befehls auf die serielle SS
	  Serial.print("Verify Bit - Bit-Adr: "); Serial.print(data[2] & 0x07);
	  Serial.print("     State: "); Serial.println((data[2] >> 3) & 0x01);

      if (bitRead(readbyte, data[2] & B111) == ((data[2] >> 3) & B1)) {
		notifyCVAck();
		//servo1.attach(ServoPin1,ServoMinMillis,ServoMaxMillis);    //attach the servo to pin
		//if (PiS1 != P1S1)
		//	servo1.write(P1S1);
		//else servo1.write(P2S1);  
		//delay(50);
		//servo1.write(PiS1);
      }
    }
    if (data[2] >> 4 == B1111) {  //Write!  
      bitWrite(readbyte, data[2] & B111, (data[2] >> 3) & B1);
      EEPROM.write(CVadr, readbyte);
    }
  }
//*/    
  // Betriebsmode, Bedienung der Weichen
  //1.3 Bestimmen der Weichenadresse
  if (((data[0] >> 6) == 0x02) && ((data[1] >> 7) == 0x01)) {  
    dccAdr = (((~data[1]) & 0x70) << 2) | (data[0] & 0x3F); //Adresse
    dccAdr = (dccAdr << 2) | ((data[1] >> 0x01) & 0x03);  //Port bestimmen, bis zu 4 möglich
    dccAdr = dccAdr + 1;
    #if defined(SHIFTADR)
    dccAdr = dccAdr - 4;
    #endif

	// Ausgabe des DCC-Befehls auf die serielle SS
	Serial.print("Normal Mode - DCC-Adr: "); Serial.print(dccAdr);
	Serial.print("     State: "); Serial.println(data[1] & B1);

    if (dccAdr == AdrServo1 && PosS1 != (data[1] & B1)) {
      timeservo1 = millis();
      servo1.attach(ServoPin1,ServoMinMillis,ServoMaxMillis);  
      servo1.write(PiS1);
      PosS1 = data[1] & B1;
      EEPROM.update(EEPROMPosS1+EEPROMSHIFT,PosS1);      
    }
    if (dccAdr == AdrServo2 && PosS2 != (data[1] & B1)) {
      timeservo2 = millis();
      servo2.attach(ServoPin2,ServoMinMillis,ServoMaxMillis);  
      servo2.write(PiS2);
      PosS2 = data[1] & B1;
      EEPROM.update(EEPROMPosS2+EEPROMSHIFT,PosS2);
    }
    if (dccAdr == AdrServo3 && PosS3 != (data[1] & B1)) {
      timeservo3 = millis();
      servo3.attach(ServoPin3,ServoMinMillis,ServoMaxMillis);  
      servo3.write(PiS3);
      PosS3 = data[1] & B1;
      EEPROM.update(EEPROMPosS3+EEPROMSHIFT,PosS3);
    }
    if (dccAdr == AdrServo4 && PosS4 != (data[1] & B1)) {
      timeservo4 = millis();
      servo4.attach(ServoPin4,ServoMinMillis,ServoMaxMillis);  
      servo4.write(PiS4);
      PosS4 = data[1] & B1;
      EEPROM.update(EEPROMPosS4+EEPROMSHIFT,PosS4);
    }
	if (dccAdr == AdrServo5 && PosS5 != (data[1] & B1)) {
		timeservo5 = millis();
		servo5.attach(ServoPin5, ServoMinMillis, ServoMaxMillis);
		servo5.write(PiS5);
		PosS5 = data[1] & B1;
		EEPROM.update(EEPROMPosS5 + EEPROMSHIFT, PosS5);
	}
	if (dccAdr == AdrServo6 && PosS6 != (data[1] & B1)) {
		timeservo6 = millis();
		servo6.attach(ServoPin6, ServoMinMillis, ServoMaxMillis);
		servo6.write(PiS6);
		PosS6 = data[1] & B1;
		EEPROM.update(EEPROMPosS6 + EEPROMSHIFT, PosS6);
	}
	if (dccAdr == AdrServo7 && PosS7 != (data[1] & B1)) {
		timeservo7 = millis();
		servo7.attach(ServoPin7, ServoMinMillis, ServoMaxMillis);
		servo7.write(PiS7);
		PosS7 = data[1] & B1;
		EEPROM.update(EEPROMPosS7 + EEPROMSHIFT, PosS7);
	}
	if (dccAdr == AdrServo8 && PosS8 != (data[1] & B1)) {
		timeservo8 = millis();
		servo8.attach(ServoPin8, ServoMinMillis, ServoMaxMillis);
		servo8.write(PiS8);
		PosS8 = data[1] & B1;
		EEPROM.update(EEPROMPosS8 + EEPROMSHIFT, PosS8);
	}
	if (dccAdr == AdrServo9 && PosS9 != (data[1] & B1)) {
		timeservo9 = millis();
		servo9.attach(ServoPin9, ServoMinMillis, ServoMaxMillis);
		servo9.write(PiS9);
		PosS9 = data[1] & B1;
		EEPROM.update(EEPROMPosS9 + EEPROMSHIFT, PosS9);
	}
	if (dccAdr == AdrServo10 && PosS10 != (data[1] & B1)) {
		timeservo10 = millis();
		servo10.attach(ServoPin10, ServoMinMillis, ServoMaxMillis);
		servo10.write(PiS10);
		PosS10 = data[1] & B1;
		EEPROM.update(EEPROMPosS10 + EEPROMSHIFT, PosS10);
	}
	if (dccAdr == AdrServo11 && PosS11 != (data[1] & B1)) {
		timeservo11 = millis();
		servo11.attach(ServoPin11, ServoMinMillis, ServoMaxMillis);
		servo11.write(PiS11);
		PosS11 = data[1] & B1;
		EEPROM.update(EEPROMPosS11 + EEPROMSHIFT, PosS11);
	}
	if (dccAdr == AdrServo12 && PosS12 != (data[1] & B1)) {
		timeservo12 = millis();
		servo12.attach(ServoPin12, ServoMinMillis, ServoMaxMillis);
		servo12.write(PiS12);
		PosS12 = data[1] & B1;
		EEPROM.update(EEPROMPosS12 + EEPROMSHIFT, PosS12);
	}
	if (dccAdr == AdrServo13 && PosS13 != (data[1] & B1)) {
		timeservo13 = millis();
		servo13.attach(ServoPin13, ServoMinMillis, ServoMaxMillis);
		servo13.write(PiS13);
		PosS13 = data[1] & B1;
		EEPROM.update(EEPROMPosS13 + EEPROMSHIFT, PosS13);
	}
	if (dccAdr == AdrServo14 && PosS14 != (data[1] & B1)) {
		timeservo14 = millis();
		servo14.attach(ServoPin14, ServoMinMillis, ServoMaxMillis);
		servo14.write(PiS14);
		PosS14 = data[1] & B1;
		EEPROM.update(EEPROMPosS14 + EEPROMSHIFT, PosS14);
	}
	if (dccAdr == AdrServo15 && PosS15 != (data[1] & B1)) {
		timeservo15 = millis();
		servo15.attach(ServoPin15, ServoMinMillis, ServoMaxMillis);
		servo15.write(PiS15);
		PosS15 = data[1] & B1;
		EEPROM.update(EEPROMPosS15 + EEPROMSHIFT, PosS15);
	}
	//if (dccAdr == AdrServo16 && PosS16 != (data[1] & B1)) {
	//	timeservo16 = millis();
	//	servo16.attach(ServoPin16, ServoMinMillis, ServoMaxMillis);
	//	servo16.write(PiS16);
	//	PosS16 = data[1] & B1;
	//	EEPROM.update(EEPROMPosS16 + EEPROMSHIFT, PosS16);
	//}


  }
}

//--------------------------------------------------------------------------------
void EEPROMReadToRam() {
  #if defined(UNLOCKEEPROM)
  //Decoder has no values or reset - set default values!
   #if defined(LOCKCVPROG)
   if (EEPROM.read(200) == 0xFF && ProgMode > 0) 
   {
   #else
   if (EEPROM.read(200) == 0xFF) { 
   #endif 
    EEPROM.update(200,0x0F);
	
    EEPROM.update(EEPROMCVMinMillis+EEPROMSHIFT, ServoMinMillis / MultiplicateMillis);
    EEPROM.update(EEPROMCVMaxMillis+EEPROMSHIFT, ServoMaxMillis / MultiplicateMillis);
    //----------- Servo 1 -------------------------------
    EEPROM.update(EEPROMAdrH1+EEPROMSHIFT, AdrServo1 >> 8);
    EEPROM.update(EEPROMAdrL1+EEPROMSHIFT, AdrServo1 & 0xFF);
    EEPROM.update(EEPROMP1S1+EEPROMSHIFT, P1S1);
    EEPROM.update(EEPROMP2S1+EEPROMSHIFT, P2S1);
    EEPROM.update(EEPROMAcS1+EEPROMSHIFT, AcS1);
    EEPROM.update(EEPROMPosS1+EEPROMSHIFT, PosS1);
    EEPROM.update(EEPROMSpP1S1+EEPROMSHIFT, SpP1S1);
    EEPROM.update(EEPROMSpP2S1+EEPROMSHIFT, SpP2S1);
    //----------- Servo 2 -------------------------------
    EEPROM.update(EEPROMAdrH2+EEPROMSHIFT, AdrServo2 >> 8);
    EEPROM.update(EEPROMAdrL2+EEPROMSHIFT, AdrServo2 & 0xFF);
    EEPROM.update(EEPROMP1S2+EEPROMSHIFT, P1S2);
    EEPROM.update(EEPROMP2S2+EEPROMSHIFT, P2S2);
    EEPROM.update(EEPROMAcS2+EEPROMSHIFT, AcS2);
    EEPROM.update(EEPROMPosS2+EEPROMSHIFT, PosS2);
    EEPROM.update(EEPROMSpP1S2+EEPROMSHIFT, SpP1S2);
    EEPROM.update(EEPROMSpP2S2+EEPROMSHIFT, SpP2S2);
    //----------- Servo 3 -------------------------------
    EEPROM.update(EEPROMAdrH3+EEPROMSHIFT, AdrServo3 >> 8);
    EEPROM.update(EEPROMAdrL3+EEPROMSHIFT, AdrServo3 & 0xFF);
    EEPROM.update(EEPROMP1S3+EEPROMSHIFT, P1S3);
    EEPROM.update(EEPROMP2S3+EEPROMSHIFT, P2S3);
    EEPROM.update(EEPROMAcS3+EEPROMSHIFT, AcS3);
    EEPROM.update(EEPROMPosS3+EEPROMSHIFT, PosS3);
    EEPROM.update(EEPROMSpP1S3+EEPROMSHIFT, SpP1S3);
    EEPROM.update(EEPROMSpP2S3+EEPROMSHIFT, SpP2S3);
    //----------- Servo 4 -------------------------------
    EEPROM.update(EEPROMAdrH4+EEPROMSHIFT, AdrServo4 >> 8);
    EEPROM.update(EEPROMAdrL4+EEPROMSHIFT, AdrServo4 & 0xFF);
    EEPROM.update(EEPROMP1S4+EEPROMSHIFT, P1S4);
    EEPROM.update(EEPROMP2S4+EEPROMSHIFT, P2S4);
    EEPROM.update(EEPROMAcS4+EEPROMSHIFT, AcS4);
    EEPROM.update(EEPROMPosS4+EEPROMSHIFT, PosS4);
    EEPROM.update(EEPROMSpP1S4+EEPROMSHIFT, SpP1S4);
    EEPROM.update(EEPROMSpP2S4+EEPROMSHIFT, SpP2S4);
	//----------- Servo 5 -------------------------------
	EEPROM.update(EEPROMAdrH5 + EEPROMSHIFT, AdrServo5 >> 8);
	EEPROM.update(EEPROMAdrL5 + EEPROMSHIFT, AdrServo5 & 0xFF);
	EEPROM.update(EEPROMP1S5 + EEPROMSHIFT, P1S5);
	EEPROM.update(EEPROMP2S5 + EEPROMSHIFT, P2S5);
	EEPROM.update(EEPROMAcS5 + EEPROMSHIFT, AcS5);
	EEPROM.update(EEPROMPosS5 + EEPROMSHIFT, PosS5);
	EEPROM.update(EEPROMSpP1S5 + EEPROMSHIFT, SpP1S5);
	EEPROM.update(EEPROMSpP2S5 + EEPROMSHIFT, SpP2S5);
	//----------- Servo 6 -------------------------------
	EEPROM.update(EEPROMAdrH6 + EEPROMSHIFT, AdrServo6 >> 8);
	EEPROM.update(EEPROMAdrL6 + EEPROMSHIFT, AdrServo6 & 0xFF);
	EEPROM.update(EEPROMP1S6 + EEPROMSHIFT, P1S6);
	EEPROM.update(EEPROMP2S6 + EEPROMSHIFT, P2S6);
	EEPROM.update(EEPROMAcS6 + EEPROMSHIFT, AcS6);
	EEPROM.update(EEPROMPosS6 + EEPROMSHIFT, PosS6);
	EEPROM.update(EEPROMSpP1S6 + EEPROMSHIFT, SpP1S6);
	EEPROM.update(EEPROMSpP2S6 + EEPROMSHIFT, SpP2S6);
	//----------- Servo 7 -------------------------------
	EEPROM.update(EEPROMAdrH7 + EEPROMSHIFT, AdrServo7 >> 8);
	EEPROM.update(EEPROMAdrL7 + EEPROMSHIFT, AdrServo7 & 0xFF);
	EEPROM.update(EEPROMP1S7 + EEPROMSHIFT, P1S7);
	EEPROM.update(EEPROMP2S7 + EEPROMSHIFT, P2S7);
	EEPROM.update(EEPROMAcS7 + EEPROMSHIFT, AcS7);
	EEPROM.update(EEPROMPosS7 + EEPROMSHIFT, PosS7);
	EEPROM.update(EEPROMSpP1S7 + EEPROMSHIFT, SpP1S7);
	EEPROM.update(EEPROMSpP2S7 + EEPROMSHIFT, SpP2S7);
	//----------- Servo 8 -------------------------------
	EEPROM.update(EEPROMAdrH8 + EEPROMSHIFT, AdrServo8 >> 8);
	EEPROM.update(EEPROMAdrL8 + EEPROMSHIFT, AdrServo8 & 0xFF);
	EEPROM.update(EEPROMP1S8 + EEPROMSHIFT, P1S8);
	EEPROM.update(EEPROMP2S8 + EEPROMSHIFT, P2S8);
	EEPROM.update(EEPROMAcS8 + EEPROMSHIFT, AcS8);
	EEPROM.update(EEPROMPosS8 + EEPROMSHIFT, PosS8);
	EEPROM.update(EEPROMSpP1S8 + EEPROMSHIFT, SpP1S8);
	EEPROM.update(EEPROMSpP2S8 + EEPROMSHIFT, SpP2S8);
	//----------- Servo 9 -------------------------------
	EEPROM.update(EEPROMAdrH9 + EEPROMSHIFT, AdrServo9 >> 8);
	EEPROM.update(EEPROMAdrL9 + EEPROMSHIFT, AdrServo9 & 0xFF);
	EEPROM.update(EEPROMP1S9 + EEPROMSHIFT, P1S9);
	EEPROM.update(EEPROMP2S9 + EEPROMSHIFT, P2S9);
	EEPROM.update(EEPROMAcS9 + EEPROMSHIFT, AcS9);
	EEPROM.update(EEPROMPosS9 + EEPROMSHIFT, PosS9);
	EEPROM.update(EEPROMSpP1S9 + EEPROMSHIFT, SpP1S9);
	EEPROM.update(EEPROMSpP2S9 + EEPROMSHIFT, SpP2S9);
	//----------- Servo 10 -------------------------------
	EEPROM.update(EEPROMAdrH10 + EEPROMSHIFT, AdrServo10 >> 8);
	EEPROM.update(EEPROMAdrL10 + EEPROMSHIFT, AdrServo10 & 0xFF);
	EEPROM.update(EEPROMP1S10 + EEPROMSHIFT, P1S10);
	EEPROM.update(EEPROMP2S10 + EEPROMSHIFT, P2S10);
	EEPROM.update(EEPROMAcS10 + EEPROMSHIFT, AcS10);
	EEPROM.update(EEPROMPosS10 + EEPROMSHIFT, PosS10);
	EEPROM.update(EEPROMSpP1S10 + EEPROMSHIFT, SpP1S10);
	EEPROM.update(EEPROMSpP2S10 + EEPROMSHIFT, SpP2S10);
	//----------- Servo 11 -------------------------------
	EEPROM.update(EEPROMAdrH11 + EEPROMSHIFT, AdrServo11 >> 8);
	EEPROM.update(EEPROMAdrL11 + EEPROMSHIFT, AdrServo11 & 0xFF);
	EEPROM.update(EEPROMP1S11 + EEPROMSHIFT, P1S11);
	EEPROM.update(EEPROMP2S11 + EEPROMSHIFT, P2S11);
	EEPROM.update(EEPROMAcS11 + EEPROMSHIFT, AcS11);
	EEPROM.update(EEPROMPosS11 + EEPROMSHIFT, PosS11);
	EEPROM.update(EEPROMSpP1S11 + EEPROMSHIFT, SpP1S11);
	EEPROM.update(EEPROMSpP2S11 + EEPROMSHIFT, SpP2S11);
	//----------- Servo 12 -------------------------------
	EEPROM.update(EEPROMAdrH12 + EEPROMSHIFT, AdrServo12 >> 8);
	EEPROM.update(EEPROMAdrL12 + EEPROMSHIFT, AdrServo12 & 0xFF);
	EEPROM.update(EEPROMP1S12 + EEPROMSHIFT, P1S12);
	EEPROM.update(EEPROMP2S12 + EEPROMSHIFT, P2S12);
	EEPROM.update(EEPROMAcS12 + EEPROMSHIFT, AcS12);
	EEPROM.update(EEPROMPosS12 + EEPROMSHIFT, PosS12);
	EEPROM.update(EEPROMSpP1S12 + EEPROMSHIFT, SpP1S12);
	EEPROM.update(EEPROMSpP2S12 + EEPROMSHIFT, SpP2S12);
	//----------- Servo 13 -------------------------------
	EEPROM.update(EEPROMAdrH13 + EEPROMSHIFT, AdrServo13 >> 8);
	EEPROM.update(EEPROMAdrL13 + EEPROMSHIFT, AdrServo13 & 0xFF);
	EEPROM.update(EEPROMP1S13 + EEPROMSHIFT, P1S13);
	EEPROM.update(EEPROMP2S13 + EEPROMSHIFT, P2S13);
	EEPROM.update(EEPROMAcS13 + EEPROMSHIFT, AcS13);
	EEPROM.update(EEPROMPosS13 + EEPROMSHIFT, PosS13);
	EEPROM.update(EEPROMSpP1S13 + EEPROMSHIFT, SpP1S13);
	EEPROM.update(EEPROMSpP2S13 + EEPROMSHIFT, SpP2S13);
	//----------- Servo 14 -------------------------------
	EEPROM.update(EEPROMAdrH14 + EEPROMSHIFT, AdrServo14 >> 8);
	EEPROM.update(EEPROMAdrL14 + EEPROMSHIFT, AdrServo14 & 0xFF);
	EEPROM.update(EEPROMP1S14 + EEPROMSHIFT, P1S14);
	EEPROM.update(EEPROMP2S14 + EEPROMSHIFT, P2S14);
	EEPROM.update(EEPROMAcS14 + EEPROMSHIFT, AcS14);
	EEPROM.update(EEPROMPosS14 + EEPROMSHIFT, PosS14);
	EEPROM.update(EEPROMSpP1S14 + EEPROMSHIFT, SpP1S14);
	EEPROM.update(EEPROMSpP2S14 + EEPROMSHIFT, SpP2S14);
	//----------- Servo 15 -------------------------------
	EEPROM.update(EEPROMAdrH15 + EEPROMSHIFT, AdrServo15 >> 8);
	EEPROM.update(EEPROMAdrL15 + EEPROMSHIFT, AdrServo15 & 0xFF);
	EEPROM.update(EEPROMP1S15 + EEPROMSHIFT, P1S15);
	EEPROM.update(EEPROMP2S15 + EEPROMSHIFT, P2S15);
	EEPROM.update(EEPROMAcS15 + EEPROMSHIFT, AcS15);
	EEPROM.update(EEPROMPosS15 + EEPROMSHIFT, PosS15);
	EEPROM.update(EEPROMSpP1S15 + EEPROMSHIFT, SpP1S15);
	EEPROM.update(EEPROMSpP2S15 + EEPROMSHIFT, SpP2S15);
	//----------- Servo 16 -------------------------------
	EEPROM.update(EEPROMAdrH16 + EEPROMSHIFT, AdrServo16 >> 8);
	EEPROM.update(EEPROMAdrL16 + EEPROMSHIFT, AdrServo16 & 0xFF);
	EEPROM.update(EEPROMP1S16 + EEPROMSHIFT, P1S16);
	EEPROM.update(EEPROMP2S16 + EEPROMSHIFT, P2S16);
	EEPROM.update(EEPROMAcS16 + EEPROMSHIFT, AcS16);
	EEPROM.update(EEPROMPosS16 + EEPROMSHIFT, PosS16);
	EEPROM.update(EEPROMSpP1S16 + EEPROMSHIFT, SpP1S16);
	EEPROM.update(EEPROMSpP2S16 + EEPROMSHIFT, SpP2S16);
	
   }
  #endif

  //--------- read Servo 1 ---------------------------
  AdrServo1 = (EEPROM.read(EEPROMAdrH1+EEPROMSHIFT) << 8) + EEPROM.read(EEPROMAdrL1+EEPROMSHIFT);
  P1S1 = checkmax(EEPROM.read(EEPROMP1S1+EEPROMSHIFT));
  P2S1 = checkmax(EEPROM.read(EEPROMP2S1+EEPROMSHIFT));
  AcS1 = EEPROM.read(EEPROMAcS1+EEPROMSHIFT);
  PosS1 = EEPROM.read(EEPROMPosS1+EEPROMSHIFT);
  SpP1S1 = EEPROM.read(EEPROMSpP1S1+EEPROMSHIFT);
  SpP2S1 = EEPROM.read(EEPROMSpP2S1+EEPROMSHIFT);
  //--------- read Servo 2 ---------------------------
  AdrServo2 = (EEPROM.read(EEPROMSHIFT+EEPROMAdrH2) << 8) + EEPROM.read(EEPROMSHIFT+EEPROMAdrL2);
  P1S2 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP1S2));
  P2S2 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP2S2));
  AcS2 = EEPROM.read(EEPROMSHIFT+EEPROMAcS2);
  PosS2 = EEPROM.read(EEPROMSHIFT+EEPROMPosS2);
  SpP1S2 = EEPROM.read(EEPROMSHIFT+EEPROMSpP1S2);
  SpP2S2 = EEPROM.read(EEPROMSHIFT+EEPROMSpP2S2);
  //--------- read Servo 3 ---------------------------
  AdrServo3 = (EEPROM.read(EEPROMSHIFT+EEPROMAdrH3) << 8) + EEPROM.read(EEPROMSHIFT+EEPROMAdrL3);
  P1S3 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP1S3));
  P2S3 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP2S3));
  AcS3 = EEPROM.read(EEPROMSHIFT+EEPROMAcS3);
  PosS3 = EEPROM.read(EEPROMSHIFT+EEPROMPosS3);
  SpP1S3 = EEPROM.read(EEPROMSHIFT+EEPROMSpP1S3);
  SpP2S3 = EEPROM.read(EEPROMSHIFT+EEPROMSpP2S3);
  //--------- read Servo 4 ---------------------------
  AdrServo4 = (EEPROM.read(EEPROMSHIFT+EEPROMAdrH4) << 8) + EEPROM.read(EEPROMSHIFT+EEPROMAdrL4);
  P1S4 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP1S4));
  P2S4 = checkmax(EEPROM.read(EEPROMSHIFT+EEPROMP2S4));
  AcS4 = EEPROM.read(EEPROMSHIFT+EEPROMAcS4);
  PosS4 = EEPROM.read(EEPROMSHIFT+EEPROMPosS4);
  SpP1S4 = EEPROM.read(EEPROMSHIFT+EEPROMSpP1S4);
  SpP2S4 = EEPROM.read(EEPROMSHIFT+EEPROMSpP2S4);
  //--------- read Servo 5 ---------------------------
  AdrServo5 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH5) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL5);
  P1S5 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S5));
  P2S5 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S5));
  AcS5 = EEPROM.read(EEPROMSHIFT + EEPROMAcS5);
  PosS5 = EEPROM.read(EEPROMSHIFT + EEPROMPosS5);
  SpP1S5 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S5);
  SpP2S5 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S5);
  //--------- read Servo 6 ---------------------------
  AdrServo6 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH6) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL6);
  P1S6 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S6));
  P2S6 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S6));
  AcS6 = EEPROM.read(EEPROMSHIFT + EEPROMAcS6);
  PosS6 = EEPROM.read(EEPROMSHIFT + EEPROMPosS6);
  SpP1S6 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S6);
  SpP2S6 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S6);
  //--------- read Servo 7 ---------------------------
  AdrServo7 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH7) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL7);
  P1S7 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S7));
  P2S7 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S7));
  AcS7 = EEPROM.read(EEPROMSHIFT + EEPROMAcS7);
  PosS7 = EEPROM.read(EEPROMSHIFT + EEPROMPosS7);
  SpP1S7 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S7);
  SpP2S7 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S7);
  //--------- read Servo 8 ---------------------------
  AdrServo8 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH8) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL8);
  P1S8 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S8));
  P2S8 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S8));
  AcS8 = EEPROM.read(EEPROMSHIFT + EEPROMAcS8);
  PosS8 = EEPROM.read(EEPROMSHIFT + EEPROMPosS8);
  SpP1S8 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S8);
  SpP2S8 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S8);
  //--------- read Servo 9 ---------------------------
  AdrServo9 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH9) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL9);
  P1S9 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S9));
  P2S9 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S9));
  AcS9 = EEPROM.read(EEPROMSHIFT + EEPROMAcS9);
  PosS9 = EEPROM.read(EEPROMSHIFT + EEPROMPosS9);
  SpP1S9 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S9);
  SpP2S9 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S9);
  //--------- read Servo 10 ---------------------------
  AdrServo10 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH10) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL10);
  P1S10 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S10));
  P2S10 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S10));
  AcS10 = EEPROM.read(EEPROMSHIFT + EEPROMAcS10);
  PosS10 = EEPROM.read(EEPROMSHIFT + EEPROMPosS10);
  SpP1S10 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S10);
  SpP2S10 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S10);
  //--------- read Servo 11 ---------------------------
  AdrServo11 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH11) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL11);
  P1S11 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S11));
  P2S11 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S11));
  AcS11 = EEPROM.read(EEPROMSHIFT + EEPROMAcS11);
  PosS11 = EEPROM.read(EEPROMSHIFT + EEPROMPosS11);
  SpP1S11 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S11);
  SpP2S11 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S11);
  //--------- read Servo 12 ---------------------------
  AdrServo12 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH12) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL12);
  P1S12 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S12));
  P2S12 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S12));
  AcS12 = EEPROM.read(EEPROMSHIFT + EEPROMAcS12);
  PosS12 = EEPROM.read(EEPROMSHIFT + EEPROMPosS12);
  SpP1S12 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S12);
  SpP2S12 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S12);
  //--------- read Servo 13 ---------------------------
  AdrServo13 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH13) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL13);
  P1S13 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S13));
  P2S13 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S13));
  AcS13 = EEPROM.read(EEPROMSHIFT + EEPROMAcS13);
  PosS13 = EEPROM.read(EEPROMSHIFT + EEPROMPosS13);
  SpP1S13 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S13);
  SpP2S13 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S13);
  //--------- read Servo 14 ---------------------------
  AdrServo14 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH14) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL14);
  P1S14 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S14));
  P2S14 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S14));
  AcS14 = EEPROM.read(EEPROMSHIFT + EEPROMAcS14);
  PosS14 = EEPROM.read(EEPROMSHIFT + EEPROMPosS14);
  SpP1S14 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S14);
  SpP2S14 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S14);
  //--------- read Servo 15 ---------------------------
  AdrServo15 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH15) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL15);
  P1S15 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S15));
  P2S15 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S15));
  AcS15 = EEPROM.read(EEPROMSHIFT + EEPROMAcS15);
  PosS15 = EEPROM.read(EEPROMSHIFT + EEPROMPosS15);
  SpP1S15 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S15);
  SpP2S15 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S15);
  //--------- read Servo 16 ---------------------------
  AdrServo16 = (EEPROM.read(EEPROMSHIFT + EEPROMAdrH16) << 8) + EEPROM.read(EEPROMSHIFT + EEPROMAdrL16);
  P1S16 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP1S16));
  P2S16 = checkmax(EEPROM.read(EEPROMSHIFT + EEPROMP2S16));
  AcS16 = EEPROM.read(EEPROMSHIFT + EEPROMAcS16);
  PosS16 = EEPROM.read(EEPROMSHIFT + EEPROMPosS16);
  SpP1S16 = EEPROM.read(EEPROMSHIFT + EEPROMSpP1S16);
  SpP2S16 = EEPROM.read(EEPROMSHIFT + EEPROMSpP2S16);

  ServoMinMillis = EEPROM.read(EEPROMSHIFT+EEPROMCVMinMillis) * MultiplicateMillis;
  ServoMaxMillis = EEPROM.read(EEPROMSHIFT+EEPROMCVMaxMillis) * MultiplicateMillis;
}

//--------------------------------------------------------------------------------
//Set Servo to Positon in EEPROM, select all (sel = 0) or one Servo that should be set!
void InitServo(uint8_t sel) {
   // Servos initialisieren

  if (sel == 0 || sel == 1) {
    //--------- read Servo 1 ---------------------------   
    servo1.attach(ServoPin1,ServoMinMillis,ServoMaxMillis);    //attach the servo to pin
    if (PosS1 == 1) {
      servo1.write(P1S1);
      PiS1 = P1S1;
    }
    else {
      servo1.write(P2S1);  
      PiS1 = P2S1;
    }
    timeservo1 = millis();
  }

  if (sel == 0 || sel == 2) {  
    //--------- read Servo 2 ---------------------------
    servo2.attach(ServoPin2,ServoMinMillis,ServoMaxMillis);  
    if (PosS2 == 1) {
      servo2.write(P1S2);
      PiS2 = P1S2;
    }
    else {
      servo2.write(P2S2);  
      PiS2 = P2S2;
    }
    timeservo2 = millis();
  }

  if (sel == 0 || sel == 3) {
    //--------- read Servo 3 ---------------------------
    servo3.attach(ServoPin3,ServoMinMillis,ServoMaxMillis);  
    if (PosS3 == 1) {
      servo3.write(P1S3);
      PiS3 = P1S3;
    }
    else {
      servo3.write(P2S3);  
      PiS3 = P2S3;
    }
    timeservo3 = millis();
  }
  
  if (sel == 0 || sel == 4) {
    //--------- read Servo 4 ---------------------------
    servo4.attach(ServoPin4,ServoMinMillis,ServoMaxMillis);  
    if (PosS4 == 1) {
      servo4.write(P1S4);
      PiS4 = P1S4;
    }
    else {
      servo4.write(P2S4);  
      PiS4 = P2S4;
    }
    timeservo4 = millis();
  }

  if (sel == 0 || sel == 5) {
	  //--------- read Servo 5 ---------------------------
	  servo5.attach(ServoPin5, ServoMinMillis, ServoMaxMillis);
	  if (PosS5 == 1) {
		  servo5.write(P1S5);
		  PiS5 = P1S5;
	  }
	  else {
		  servo5.write(P2S5);
		  PiS5 = P2S5;
	  }
	  timeservo5 = millis();
  }

  if (sel == 0 || sel == 6) {
	  //--------- read Servo 6 ---------------------------
	  servo6.attach(ServoPin6, ServoMinMillis, ServoMaxMillis);
	  if (PosS6 == 1) {
		  servo6.write(P1S6);
		  PiS6 = P1S6;
	  }
	  else {
		  servo6.write(P2S6);
		  PiS6 = P2S6;
	  }
	  timeservo6 = millis();
  }

  if (sel == 0 || sel == 7) {
	  //--------- read Servo 7 ---------------------------
	  servo7.attach(ServoPin7, ServoMinMillis, ServoMaxMillis);
	  if (PosS7 == 1) {
		  servo7.write(P1S7);
		  PiS7 = P1S7;
	  }
	  else {
		  servo7.write(P2S7);
		  PiS7 = P2S7;
	  }
	  timeservo7 = millis();
  }

  if (sel == 0 || sel == 8) {
	  //--------- read Servo 8 ---------------------------
	  servo8.attach(ServoPin8, ServoMinMillis, ServoMaxMillis);
	  if (PosS8 == 1) {
		  servo8.write(P1S8);
		  PiS8 = P1S8;
	  }
	  else {
		  servo8.write(P2S8);
		  PiS8 = P2S8;
	  }
	  timeservo8 = millis();
  }

  if (sel == 0 || sel == 9) {
	  //--------- read Servo 9 ---------------------------
	  servo9.attach(ServoPin9, ServoMinMillis, ServoMaxMillis);
	  if (PosS9 == 1) {
		  servo9.write(P1S9);
		  PiS9 = P1S9;
	  }
	  else {
		  servo9.write(P2S9);
		  PiS9 = P2S9;
	  }
	  timeservo9 = millis();
  }

  if (sel == 0 || sel == 10) {
	  //--------- read Servo 10 ---------------------------
	  servo10.attach(ServoPin10, ServoMinMillis, ServoMaxMillis);
	  if (PosS10 == 1) {
		  servo10.write(P1S10);
		  PiS10 = P1S10;
	  }
	  else {
		  servo10.write(P2S10);
		  PiS10 = P2S10;
	  }
	  timeservo10 = millis();
  }

  if (sel == 0 || sel == 11) {
	  //--------- read Servo 11 ---------------------------
	  servo11.attach(ServoPin11, ServoMinMillis, ServoMaxMillis);
	  if (PosS11 == 1) {
		  servo11.write(P1S11);
		  PiS11 = P1S11;
	  }
	  else {
		  servo11.write(P2S11);
		  PiS11 = P2S11;
	  }
	  timeservo11 = millis();
  }

  if (sel == 0 || sel == 12) {
	  //--------- read Servo 12 ---------------------------
	  servo12.attach(ServoPin12, ServoMinMillis, ServoMaxMillis);
	  if (PosS12 == 1) {
		  servo12.write(P1S12);
		  PiS12 = P1S12;
	  }
	  else {
		  servo12.write(P2S12);
		  PiS12 = P2S12;
	  }
	  timeservo12 = millis();
  }

  if (sel == 0 || sel == 13) {
	  //--------- read Servo 13 ---------------------------
	  servo13.attach(ServoPin13, ServoMinMillis, ServoMaxMillis);
	  if (PosS13 == 1) {
		  servo13.write(P1S13);
		  PiS13 = P1S13;
	  }
	  else {
		  servo13.write(P2S13);
		  PiS13 = P2S13;
	  }
	  timeservo13 = millis();
  }

  if (sel == 0 || sel == 14) {
	  //--------- read Servo 14 ---------------------------
	  servo14.attach(ServoPin14, ServoMinMillis, ServoMaxMillis);
	  if (PosS14 == 1) {
		  servo14.write(P1S14);
		  PiS14 = P1S14;
	  }
	  else {
		  servo14.write(P2S14);
		  PiS14 = P2S14;
	  }
	  timeservo14 = millis();
  }

  if (sel == 0 || sel == 15) {
	  //--------- read Servo 15 ---------------------------
	  servo15.attach(ServoPin15, ServoMinMillis, ServoMaxMillis);
	  if (PosS15 == 1) {
		  servo15.write(P1S15);
		  PiS15 = P1S15;
	  }
	  else {
		  servo15.write(P2S15);
		  PiS15 = P2S15;
	  }
	  timeservo15 = millis();
  }

  //if (sel == 0 || sel == 16) {
	  //--------- read Servo 16 ---------------------------
	 // servo16.attach(ServoPin16, ServoMinMillis, ServoMaxMillis);
	 // if (PosS16 == 1) {
		//  servo16.write(P1S16);
		//  PiS16 = P1S16;
	 // }
	 // else {
		//  servo16.write(P2S16);
		//  PiS16 = P2S16;
	 // }
	 // timeservo16 = millis();
  //}

  /*
  delay(500);

  if (AcS1 != 0)
    servo1.detach();
  if (AcS2 != 0)  
    servo2.detach();
  if (AcS3 != 0)
    servo3.detach();
  if (AcS4 != 0)
    servo4.detach();
    */
}

//--------------------------------------------------------------------------------
byte checkmax (byte in) {
  if (in > 180)
    return 180;
  return in;  
}

//--------------------------------------------------------------------------------
byte getNextStep(byte ist, byte soll) {
  byte value = 1;
  
  if (soll > ist) {
    ist = ist + value;
    if (ist > soll) //count over with value?
      ist = soll;
  }
  else {
    ist = ist - value;
    if (soll > ist) //count over with value?
      ist = soll;
  }
  return ist;
}

//--------------------------------------------------------------------------------
// relevanten EEPROM-Inhalt auf SS ausgeben
void dumpEEPROM(void)
{
	char ausgabestring[16];

	// Die Aufteilung mit Blanks passt wahrscheinlich nur im Monitor von VS 2017 auf die Spalten der Werte (proportionale Schrift)
	Serial.println("  CV        AdrH AdrL Pos1 Pos2 Time   Sp1 Sp2  Dummys   AktPos"); 

	for (int x = 0; x < 16; x++)								// 16 Servos
	{
		if(x < 9) Serial.print("  ");							// fuehrende Blanks bei einstelligen Zahlen
		Serial.print("CV");Serial.print(10 + x*10); Serial.print(": ");
		for (int y = 0; y < 10; y++)							// je 10 Zellen
		{
			sprintf(ausgabestring, " %6d", EEPROM.read(EEPROMSHIFT + (10 + x*10) + y));
			Serial.print(ausgabestring);

		}
		Serial.println();
	}
	sprintf(ausgabestring, "EEPROM-Flag %4d", EEPROM.read(200));
	Serial.println(ausgabestring);
}	

// This function is called by the NmraDcc library when a DCC ACK needs to be sent
// Calling this function should cause an increased 60ma current drain on the power supply for 6ms to ACK a CV Read 
void notifyCVAck(void)
{
	Serial.println("notifyCVAck");

	digitalWrite(DccAckPin, HIGH);
	delay(6);
	digitalWrite(DccAckPin, LOW);

	//servo1.attach(ServoPin1,ServoMinMillis,ServoMaxMillis);    //attach the servo to pin

	//for (byte i = 0; i < 1; i++)
	//{
	//	servo1.write(100);
	//	servo1.write(120);
	//}

	//if (PiS1 != P1S1)
	//	servo1.write(P1S1);
	//else servo1.write(P2S1);  

	delay(50);
	//servo1.write(PiS1);
	//servo1.detach();

}
